### Nested Logit model estimation

### Create the data array from the existing dataset
rm(list=ls(all=TRUE))
setwd("L:/")
Data1  <- read.csv("Data_Clean_English.csv",header=TRUE)

Data <- subset(Data1, PurposeCode==500)

## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(10) 

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Alternative specific constants
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]

  ## Travel time to destination
  d1 <- x[4]
  
  ## Fare
  f1 <- x[5]
  f2 <- x[6]
  f3 <- x[7]
  
  
  ## Distance
  ##dis1 <- x[6]
  ##dis2 <- x[7]
  
  ## Number of Changes or Transfers
  ##nf1 <- x[6]
  
  ##Accessibility
  ac1 <- x[8]
  ac2 <- x[9]
  
  ##Scale parameter
  pa <- x[10]

  ## declare the log-likelihood variable, set value to 0
  LL = 0

  ### For this choice problem, we consider the following 4 modes
  ## 鉄道(train)  - (public)
  ## バス(bus)    - (public)
  ## 自動車(car)  - (private)
  ## 自転車(bike) - (private)
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
                                        # time           # fare      # constant
  train  <- Data$ModeAvailableTrain*exp(d1*Data$TotalTimeTrain/100  +f1*Data$FareTrain/100 + ac1*(Data$AccessTimeTrain+Data$EgressTimeTrain)/100  +b1*matrix(1,nrow =hh,ncol=1))
  bus    <- Data$ModeAvailableBus  *exp(d1*Data$TotalTimeBus/100    +f1*Data$FareBus/100   + ac2*(Data$AccessTimeBus+Data$EgressTimeBus)/100                                    )
  car    <- Data$ModeAvailableCar  *exp(d1*Data$TimeCar/100         +f2*Data$DistanceCar*6/1000           +f3*Data$CBDDummy                       +b2*matrix(1,nrow =hh,ncol=1))
  bike   <- Data$ModeAvailableBike *exp(d1*Data$TimeBike/100                                                                                      +b3*matrix(1,nrow =hh,ncol=1))
 ## walk   <- Data$ModeAvailableWalk *exp(d1*Data$TimeWalk/100                                                     )

  ### Construct nests.First, calculate public or private choice probabilityP(LV1)
  ## setting of log-sum variables
  logsum.public  <- log( ( (train+bus)!=0) * (train + bus) + ((train+bus)==0))
  ##logsum.car <- log( ( (car)!=0) * (car) + ((car)==0))
  ##logsum.bike <- log( ( (bike)!=0) * (bike) + ((bike)==0))

  ##Calculate public or private choice probability with these log-sum variables
  nume.public   <- (logsum.public != 0)*exp(pa*logsum.public) + (logsum.public == 0)
  ##nume.car  <- (logsum.car != 0)*exp(pa*logsum.car) + (logsum.car == 0)
  ##ume.bike  <- (logsum.bike != 0)*exp(pa*logsum.bike) + (logsum.bike == 0)

  deno <- nume.public + car + bike
  P.public  <- nume.public  / deno
  P.car <- car / deno
  P.bike <- bike / deno


  ### Next, calculate conditional probability.P(LV2|LV1)
  ## conditional probability under public choice
  deno.public     <- train + bus
  P.train.public  <- train / ((deno.public!=0)*deno.public + (deno.public==0))
  P.bus.public    <- bus   / ((deno.public!=0)*deno.public + (deno.public==0))

  ### Finally, calculate joint probability. P(LV1, LV2) = P(LV2|LV1)*P(LV1)
  P.train <- P.train.public * P.public
  P.bus   <- P.bus.public   * P.public
 
  ## Avoid problems stemming from choice probabilities becoming zero.
  P.train <- (P.train!=0)*P.train + (P.train==0)
  P.bus   <- (P.bus!=0)  *P.bus   + (P.bus==0)
  P.car   <- (P.car!=0)  *P.car   + (P.car==0)
  P.bike  <- (P.bike!=0) *P.bike  + (P.bike==0)
 

  ## Choice results
  Ctrain   <- Data$MainModeENG =="Rail"
  Cbus     <- Data$MainModeENG =="Bus"
  Ccar     <- Data$MainModeENG =="Car"
  Cbike    <- Data$MainModeENG =="Bicycle"
  ##Cwalk    <- Data$MainModeENG =="Walk"

  ## Calculate the Log-likelihood function
  LL <- colSums(Ctrain*log(P.train) + Cbus*log(P.bus) + Ccar  *log(P.car)   + Cbike  *log(P.bike))

}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation、Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
